import 'package:chessjs/chessjs.dart' as chessjs;
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_chessboard_js/chessboard/utils.dart';
import 'chessboard_controller.dart';

/// A single square on the chessboard
/// this class receives a string with the name of the square,
/// the chessboard controller and the current size
/// The chessboard only needs to rebuild the squares involved in the move
/// for performance that's what this class handles.
/// The square from where the move originated and the square where the move finished
/// are the only ones rebuilt. An special case is when a side promotes,
/// in this case the whole chessboard rebuilds TODO fix in the future

typedef void MoveCallback(Map? move);

class BoardSquare extends StatefulWidget {
  BoardSquare({
    required Key key,
    required this.squareName,
    required this.size,
    required this.moveCallback,
    required this.controller,
  });

  final String squareName;
  final double size;
  final MoveCallback moveCallback;
  final ChessboardController controller;

  @override
  _BoardSquareState createState() => _BoardSquareState();
}

class _BoardSquareState extends State<BoardSquare> {
  late final ChessboardController controller;

  @override
  void initState() {
    controller = widget.controller;
    controller.addListener(() {
      final lastMove = controller.getVerboseHistory().last;
      final from = lastMove['from'];
      final to = lastMove['to'];
      if (widget.squareName.contains(from) || widget.squareName.contains(to))
        setState(() {});
    });
    super.initState();
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
        flex: 1,
        child: DragTarget(
          builder: (context, accepted, rejected) {
            /// get the piece on the square provided
            return controller.gameLogic.get(widget.squareName) != null
                ?

                /// if there is a piece on the square, returns a draggable image of the piece
                Draggable(
                    child: getImageToDisplay(
                        size: widget.size / 8,
                        controller: controller,
                        squareName: widget.squareName),
                    feedback: getImageToDisplay(
                        size: (1.15 * (widget.size / 8)),
                        controller: controller,
                        squareName: widget.squareName),

                    onDragCompleted: () {
                      setState(() {});
                    },

                    /// Data to be dropped in this drag target: the squareName where
                    /// the move originated, the type of the piece and the color of the piece
                    data: [
                      widget.squareName,
                      controller.gameLogic
                          .get(widget.squareName)!
                          .type
                          .toUpperCase(),
                      controller.gameLogic.get(widget.squareName)!.color,
                    ],
                  )

                /// Changing here Container for SizedBox wont allow to make a move
                /// Container expands to the child size, SizedBox wont, so the drag target
                /// property will be on a 0 size widget and wont work
                : Container();

            /// accepted information about the move
            /// the square from where the move originated is moveInfo[0],
            /// the type of piece is moveInfo[1], and the color is moveInfo[2]
          },
          onWillAccept: (List? moveInfo) {
            if (moveInfo != null) {
              /// If dragging and dropping in same place return false
              if (moveInfo[0] != widget.squareName) {
                /// moveInfo should have 3 items
                if (moveInfo.length == 3) {
                  /// Accept only if the game turn is right and the enableUserMoves is true
                  if (moveInfo[2] == controller.gameLogic.turn &&
                      controller.enableUserMoves) {
                    /// DO NOT ACCEPT ILLEGAL MOVES
                    final legalMoves = controller.gameLogic.generateMoves();
                    legalMoves.removeWhere((move) {
                      /// if [from] move part is different, remove item
                      if (move.fromAlgebraic != moveInfo[0]) {
                        return true;
                      } else {
                        /// if [to] move part is different, also remove item
                        if (move.toAlgebraic != widget.squareName) {
                          return true;
                        } else {
                          return false;
                        }
                      }
                    });

                    /// If there were not matches of intended move
                    if (legalMoves.isEmpty) {
                      return false;
                    } else {
                      return true;
                    }
                  } else {
                    return false;
                  }
                } else {
                  return false;
                }
              } else
                return false;
            } else
              return false;
          },
          onAccept: (List moveInfo) async {
            /// Special moves like promotion will rebuild the whole board by
            /// sending a callback and calling setState on the chessboard (parent widget)
            /// normal moves will only rebuilt the two squares involved which is more efficient
            Map? move;

            /// Here is accepted information about the move
            /// the square from where the move originated (moveInfo[0]),
            /// the type of piece (moveInfo[1]), and the color (moveInfo[2])

            /// Check if is a promotion move: the piece is a pawn, was in the rank 2 or 7
            /// and went to 1 or 8 (depending on the color)
            if (moveInfo[1] == "P" &&
                ((moveInfo[0][1] == "7" &&
                        widget.squareName[1] == "8" &&
                        moveInfo[2] == chessjs.WHITE) ||
                    (moveInfo[0][1] == "2" &&
                        widget.squareName[1] == "1" &&
                        moveInfo[2] == chessjs.BLACK))) {
              /// get the promotion piece selected by the user from the dialog
              final val = await promotionDialog(context, controller);
              if (val == 'n' || val == 'q' || val == 'r' || val == 'b') {
                /// move to return in case of promotion
                move = {
                  "from": moveInfo[0],
                  "to": widget.squareName,
                  "promotion": val,
                };

                /// send callback to the parent and rebuild
                widget.moveCallback(move);
              }
            }

            /// Check if is a castling move, in that
            /// case is same than promotion, the square
            /// of the rook must be forced to update
            else if (

                /// white castle
                (moveInfo[1] == "K" &&
                        moveInfo[0] == "e1" &&
                        moveInfo[2] == chessjs.WHITE &&
                        (widget.squareName == "g1" ||
                            widget.squareName == "c1")) ||

                    /// black castle
                    (moveInfo[1] == "K" &&
                        moveInfo[0] == "e8" &&
                        moveInfo[2] == chessjs.BLACK &&
                        (widget.squareName == "g8" ||
                            widget.squareName == "c8"))) {
              move = {"from": moveInfo[0], "to": widget.squareName};
              widget.moveCallback(move);
            }

            /// Check if is an en passant move, in that
            /// case is same than promotion, the square
            /// of the captured pawn must be forced to update
            else if (

                /// piece must be a pawn and the square the pawn
                /// will go after en passant must be empty
                moveInfo[1] == 'P' &&
                        widget.controller.gameLogic.get(widget.squareName) ==
                            null &&

                        /// White makes en passant
                        (

                            /// pawn goes from 5th rank to 6th rank
                            moveInfo[0][1] == '5' &&
                                moveInfo[2] == chessjs.WHITE &&
                                widget.squareName.contains('6') &&

                                /// and to one of the adjacent files
                                isAdjacentFile(
                                    widget.squareName, moveInfo[0][0])) ||

                    /// Black makes en passant
                    (

                            /// pawn goes from 4th rank to 3er rank
                            moveInfo[0][1] == '4' &&
                                moveInfo[2] == chessjs.BLACK &&
                                widget.squareName.contains('3'))

                        /// and to one of the adjacent files
                        &&
                        isAdjacentFile(widget.squareName, moveInfo[0][0])) {
              move = {"from": moveInfo[0], "to": widget.squareName};
              widget.moveCallback(move);
            }

            /// if is not a promotion just make the move
            else {
              move = {"from": moveInfo[0], "to": widget.squareName};
              setState(() {
                widget.controller.gameLogic.move(move);
                widget.controller.refreshBoard();
              });
            }
          },
        ));
  }
}
