import 'package:chessjs/chessjs.dart' as chessjs;
import 'constants.dart';
import 'package:flutter/material.dart';

typedef Null MoveCallback(Map<dynamic, dynamic> moveNotation);
typedef Null CheckMateCallback(ChessboardPieceColor color);
typedef Null CheckCallback(ChessboardPieceColor color);
typedef Null DrawCallback();

class ChessboardController extends ChangeNotifier {
  ChessboardController({
    this.onMove,
    this.onCheckMate,
    this.onCheck,
    this.onDraw,
    this.enableUserMoves = true,
    chessjs.Chess? chessjsGame,
  }) {
    if (chessjsGame == null) {
      gameLogic = chessjs.Chess();
    } else
      gameLogic = chessjsGame;
  }

  /// Creates a logical chess game
  late chessjs.Chess gameLogic;

  /// Callback for when a move is made
  MoveCallback? onMove;

  /// Callback for when a player is checkmated
  CheckMateCallback? onCheckMate;

  ///Callback for when a player is in check
  CheckCallback? onCheck;

  /// Callback for when the game is a draw (Example: K v K)
  DrawCallback? onDraw;

  /// User moves can be enabled or disabled by this property
  bool enableUserMoves;

  /// TODO make select a move possible in the notation
  /// Implement how to iterate over lines.
  /// Also create functions to display in the board
  /// the state after selecting a move from notation.
  /// The PROBLEM -> we are displaying the state of the board
  /// based on the last state of the history. In the [buildBoard] function
  /// the controller is passed to each [BoardSquare] and the
  /// [board] variable of [gameLogic] tells whether a piece in on the square

  /// Get history of game (moves) in SAN format
  /// Returns a List<String>
  List<String> getSanHistory() {
    List<String> history = gameLogic.getHistorySAN();
    return history;
  }

  /// Get history of game (moves) with details for each move.
  /// Returns a List<Map>
  List<Map> getVerboseHistory() {
    List<Map> history = gameLogic.getHistoryVerbose();
    return history;
  }

  void toggleEnableMove() {
    enableUserMoves = !enableUserMoves;
    notifyListeners();
  }

  void makeMove(String move) {
    gameLogic.move(move);
    refreshBoard();
  }

  /// Refreshes board
  void refreshBoard() {
    if (gameLogic.inCheckmate && onCheckMate != null) {
      onCheckMate!(gameLogic.turn == chessjs.WHITE
          ? ChessboardPieceColor.White
          : ChessboardPieceColor.Black);
    } else if ((gameLogic.inDraw ||
            gameLogic.inStalemate ||
            gameLogic.inThreefoldRepetition ||
            gameLogic.insufficientMaterial) &&
        onDraw != null) {
      onDraw!();
    } else if (gameLogic.inCheck && onCheck != null) {
      onCheck!(gameLogic.turn == chessjs.WHITE
          ? ChessboardPieceColor.White
          : ChessboardPieceColor.Black);
    } else {
      if (onMove != null) onMove!(gameLogic.getHistoryVerbose().last);
    }
    notifyListeners();
  }
}
